@@ -83,6 +83,7 @@ CdSensor	*cd_sensor_new			(void);
 /* accessors */
 const gchar	*cd_sensor_get_id		(CdSensor		*sensor);
 const gchar	*cd_sensor_get_object_path	(CdSensor		*sensor);
+const gchar	*cd_sensor_get_device_path	(CdSensor		*sensor);
 gboolean	 cd_sensor_register_object	(CdSensor		*sensor,
 						 GDBusConnection	*connection,
 						 GDBusInterfaceInfo	*info,
@@ -91,6 +92,8 @@ gboolean	 cd_sensor_register_object	(CdSensor		*sensor,
 gboolean	 cd_sensor_set_from_device	(CdSensor		*sensor,
 						 GUdevDevice		*device,
 						 GError			**error);
+void		 cd_sensor_set_index		(CdSensor		*sensor,
+						 guint			 idx);
 #endif
 void		 cd_sensor_button_pressed	(CdSensor		*sensor);
 gboolean	 cd_sensor_dump			(CdSensor		*sensor,
